require_relative '../spec_helper'
describe SkypeCheck::Username do

  let(:query_url) { "#{SkypeCheck.configuration.username_validator_endpoint}?new_username=username" }
  let(:successfull_response) { '{"status": 200}' }
  let(:taken_response) { '{"status":406, "data": {"markup":"Skype Name not available", "alternatives":true}}' }
  let(:available_response) { '{"status":200}' }

  let(:too_long_username) { (0..32).map {"a"}.join }
  let(:too_short_username) { 'abcde' }

  describe ".query" do
    context "when  successfull" do
      it  "should return a parsed json response as Hash" do
        stub_request(:get, query_url).to_return(status: 200, body: successfull_response, headers: {})

        expect(SkypeCheck::Username.query('username').class).to eq(Hash)
      end
    end

    context "when connection error" do 
      it "can raise SocketError" do
        SkypeCheck::Username.should_receive(:query).with('username').and_raise(SocketError)
        lambda { SkypeCheck::Username.query('username')}.should raise_error(SocketError)
      end
    end

    context "when username is not valid" do
      it "should raise UsernameValidationError with username that is longer than 32 chars" do
        expect{ SkypeCheck::Username.query(too_long_username) }.to raise_error(SkypeCheck::UsernameValidationError, "Input username is too long!")
      end

      it "should raise UsernameValidationError with username that is shorter than 6 chars" do
        expect{ SkypeCheck::Username.query(too_short_username) }.to raise_error(SkypeCheck::UsernameValidationError, "Input username is too short!")
      end

      it "should raise UsernameValidationError with username that begins with a number" do
        expect{ SkypeCheck::Username.query('1_username') }.to raise_error(SkypeCheck::UsernameValidationError, "Input username should not start with a number!")
      end


      it "should raise UsernameValidationError when username is nil" do
        expect{ SkypeCheck::Username.query(nil) }.to raise_error(SkypeCheck::UsernameValidationError, "Input username is nil!")
      end

    end
  end

  describe ".is_taken?" do
    it "should return true if username is already taken" do
      stub_request(:get, query_url).to_return(status: 200, body: taken_response, headers: {})

      expect(SkypeCheck::Username.is_taken?('username')).to eq(true)
    end
  end

  describe ".is_available?" do
    it "should return true if username is available" do
      stub_request(:get, query_url).
        to_return(status: 200, body: available_response, headers: {})

      expect(SkypeCheck::Username.is_available?('username')).to eq(true)
    end
  end
      
end
